from urllib.parse import urljoin

from bs4 import BeautifulSoup

from .core import Check
from .. import fetch
from ..issues import Issues, Issue
from .registry import register
from ..urls import href_is_url


@register
class BrokenImageCheck(Check):
    def run(self):
        issues = Issues()

        soup = BeautifulSoup(self.content, "html.parser")
        for img in soup.findAll("img"):
            src = img.get("src")
            full_src = urljoin(self.url, src)

            try:
                response = fetch.head(full_src)
                if response.status_code == 405:
                    # just skip for now if HEAD not allowed
                    continue
                response.raise_for_status()
            except Exception as e:
                issues.append(
                    Issue.from_type(
                        type="img-src-broken",
                        url=self.url,
                        extra={"src": full_src, "exception": str(e)},
                    )
                )

        return issues


@register
class BrokenLinkCheck(Check):
    def run(self):
        issues = Issues()

        soup = BeautifulSoup(self.content, "html.parser")
        for a in soup.findAll("a"):
            href = a.get("href")
            if not href:
                issues.append(
                    Issue.from_type(
                        type="a-without-href", url=self.url, extra={"element": str(a)}
                    )
                )
                continue

            if not href_is_url(href):
                continue

            full_href = urljoin(self.url, href)

            try:
                response = fetch.head(full_href)
                if not response.ok:
                    get_response = fetch.get(full_href)
                    get_response.raise_for_status()
            except Exception as e:
                # TODO if not allowed status returned, change to warning?
                issues.append(
                    Issue.from_type(
                        type="a-href-broken",
                        url=self.url,
                        extra={"element": str(a), "exception": str(e)},
                    )
                )

        return issues
