from urllib.parse import urljoin, urlparse

from bs4 import BeautifulSoup

from .. import fetch
from .core import Check
from ..issues import Issues, Issue
from .registry import register
from ..urls import url_is_https, url_is_local


@register
class HttpsUrlCheck(Check):
    def run(self):
        issues = Issues()

        if url_is_https(self.url):
            http_url = "http" + self.url[5:]
            response = fetch.head(http_url)
            if not response.ok:
                issues.append(
                    Issue.from_type(
                        type="http-url-broken", url=self.url, extra={"url": self.url}
                    )
                )
            elif not url_is_https(response.url):
                issues.append(
                    Issue.from_type(
                        type="https-redirect-missing",
                        url=self.url,
                        extra={"url": self.url},
                    )
                )
        elif not url_is_local(self.url):
            issues.append(
                Issue.from_type(type="http-url", url=self.url, extra={"url": self.url})
            )

        return issues


# ideally this would use chromium/puppeteer so that we just
# leverage chrome's decisions on this kind of thing
@register
class MixedContentCheck(Check):
    def run(self):
        issues = Issues()

        if not url_is_https(self.url):
            # if this page is not https, it doesn't matter
            return issues

        soup = BeautifulSoup(self.content, "html.parser")

        to_check = {
            "img": {"attr": "src"},
            "link": {"attr": "href", "ignore": {"rel": "profile"}},
            "iframe": {"attr": "src"},
        }

        for type, cfg in to_check.items():
            for el in soup.findAll(type):
                if any(
                    [
                        el.get(k, [None])[0] == v
                        for k, v in cfg.get("ignore", {}).items()
                    ]
                ):
                    continue

                attr = cfg["attr"]

                value = el.get(attr)
                full_value = urljoin(self.url, value)

                if not url_is_https(full_value):
                    issues.append(
                        Issue.from_type(
                            type="https-mixed-content",
                            url=self.url,
                            extra={"element": str(el)},
                        )
                    )

        return issues
