import re

from bs4 import BeautifulSoup

from .core import Check
from .registry import register
from ..issues import Issues, Issue
from .. import fetch
from ..urls import url_is_root, url_is_local


class BaseOpenGraphCheck(Check):
    # og_property name (without og:)
    def run(self):
        issues = Issues()

        self.soup = BeautifulSoup(self.content, "html.parser")

        meta = self.soup.find("meta", {"property": f"og:{self.og_property}"})
        self.meta_tag_content = meta.get("content", "") if meta else ""
        if not self.meta_tag_content:
            property_slug = self.og_property.replace("_", "-")
            issues.append(
                Issue.from_type(
                    type=f"open-graph-{property_slug}-missing",
                    url=self.url,
                    extra={"url": self.url},
                )
            )

        return issues


@register
class OpenGraphTitleCheck(BaseOpenGraphCheck):
    og_property = "title"


@register
class OpenGraphDescriptionCheck(BaseOpenGraphCheck):
    og_property = "description"

    def run(self):
        issues = super().run()
        meta = self.soup.find("meta", {"name": "description"})
        if meta and meta.get("content", ""):
            # if there is a meta description, that is a fine alternative
            # to the more specific og:description
            issues = Issues()
        return issues


@register
class OpenGraphTypeCheck(BaseOpenGraphCheck):
    og_property = "type"


@register
class OpenGraphUrlCheck(BaseOpenGraphCheck):
    og_property = "url"

    def run(self):
        issues = super().run()

        url = self.meta_tag_content

        if url:
            try:
                response = fetch.head(url)
                if response.status_code != 405:
                    # just skip for now if HEAD not allowed
                    response.raise_for_status()
            except Exception as e:
                issues.append(
                    Issue.from_type(
                        type="open-graph-url-broken",
                        url=self.url,
                        extra={"exception": str(e)},
                    )
                )

            if not url.startswith("http://") or not (
                url.startswith("https://") or url_is_local(url)
            ):
                issues.append(
                    Issue.from_type(
                        type="open-graph-url-not-canonical",
                        url=self.url,
                        extra={"content": url},
                    )
                )

        return issues


@register
class OpenGraphImageCheck(BaseOpenGraphCheck):
    og_property = "image"

    def run(self):
        issues = super().run()

        url = self.meta_tag_content

        if url:
            try:
                response = fetch.head(url)
                if response.status_code != 405:
                    # just skip for now if HEAD not allowed
                    response.raise_for_status()
            except Exception as e:
                issues.append(
                    Issue.from_type(
                        type="open-graph-image-broken",
                        url=self.url,
                        extra={"exception": str(e)},
                    )
                )

            if not url.startswith("http://") or not (
                url.startswith("https://") or url_is_local(url)
            ):
                issues.append(
                    Issue.from_type(
                        type="open-graph-image-not-canonical",
                        url=self.url,
                        extra={"content": url},
                    )
                )

        return issues


@register
class OpenGraphSiteNameCheck(BaseOpenGraphCheck):
    og_property = "site_name"


# description warning?
# url and image should be absolute - check exists too? what if running pre-deploy... that's not a scenario right now thogh (could add condition for that)
