import re

from bs4 import BeautifulSoup

from .core import Check
from .registry import register
from ..issues import Issues, Issue
from ..urls import url_is_root
from ..settings import Setting, Settings


@Settings.register
class TitleSuffixSetting(Setting):
    name = "title-suffix"
    default = ""


_titles_seen = set()


@register
class PageTitlesCheck(Check):
    def run(self):
        issues = Issues()

        soup = BeautifulSoup(self.content, "html.parser")
        title = soup.title

        if title and title.string:
            title = title.string

            if title in _titles_seen:
                issues.append(
                    Issue.from_type(
                        type="duplicate-title",
                        url=self.url,
                        extra={"title": title, "url": self.url},
                    )
                )
            else:
                _titles_seen.add(title)

            if len(title.split()) < 2 and not url_is_root(self.url):
                issues.append(
                    Issue.from_type(
                        type="short-title",
                        url=self.url,
                        extra={"title": title, "url": self.url},
                    )
                )

            suffix = Settings["title-suffix"]
            if suffix and title.endswith(suffix):
                title = title[: -len(suffix)]

            if len(title) > 60:
                issues.append(
                    Issue.from_type(
                        type="long-title", url=self.url, extra={"title": title}
                    )
                )

            # TODO also check title-empty here? suffix used but template forgot to put something before it...

        else:
            issues.append(
                Issue.from_type(
                    type="title-missing", url=self.url, extra={"url": self.url}
                )
            )

        return issues


@register
class PageDescriptionCheck(Check):
    def run(self):
        issues = Issues()

        soup = BeautifulSoup(self.content, "html.parser")

        meta = soup.find("meta", {"name": "description"})
        if not meta:
            # missing meta is fine for google, except for social if no og:description (checked elsewhere)
            return issues

        description = meta.get("content", "")
        if not description:
            issues.append(
                Issue.from_type(
                    type="meta-description-empty",
                    url=self.url,
                    extra={"url": self.url, "element": str(meta)},
                )
            )
            return issues

        if len(description) > 320:
            issues.append(
                Issue.from_type(
                    type="long-meta-description",
                    url=self.url,
                    extra={"url": self.url, "description": description},
                )
            )

        if len(description) < 50:
            issues.append(
                Issue.from_type(
                    type="short-meta-description",
                    url=self.url,
                    extra={"url": self.url, "description": description},
                )
            )

        return issues


@register
class ImgAltCheck(Check):

    ignore_src_patterns = (
        # re.compile("^https://www.facebook.com/tr\?id=\d+"),
        # re.compile("^https://fast.wistia.com/.*swatch.*"),
    )

    def run(self):
        issues = Issues()

        soup = BeautifulSoup(self.content, "html.parser")
        for img in soup.findAll("img"):
            alt = img.get("alt")
            if alt is None:
                issues.append(
                    Issue.from_type(
                        type="img-alt-missing",
                        url=self.url,
                        extra={"element": str(img)},
                    )
                )
            # elif not alt.strip():

            #     if any(
            #         [
            #             pattern.match(img.get("src", ""))
            #             for pattern in self.ignore_src_patterns
            #         ]
            #     ):
            #         continue

            #     issues.append(
            #         Issue.from_type(
            #             type="img-alt-empty", url=self.url, extra={"element": str(img)}
            #         )
            #     )

        return issues
