import logging
import json
import os
from threading import Thread
from subprocess import run

import click
import yaml

from .runner import Runner
from .crawler import Crawler, DEFAULT_CRAWL_LIMIT, DEFAULT_CRAWL_THRESHOLD
from .server import Server
from .settings import Settings, Setting
from .report import Report
from .config import ConfigSchema


@Settings.register
class CrawlIgnore(Setting):
    name = "crawl-ignore"
    default = []


@Settings.register
class CrawlLimit(Setting):
    name = "crawl-limit"
    default = DEFAULT_CRAWL_LIMIT


@Settings.register
class CrawlThreshold(Setting):
    name = "crawl-threshold"
    default = DEFAULT_CRAWL_THRESHOLD


@click.group()
def cli():
    pass


@cli.command()
@click.argument("url")
@click.option("--fingerprint", "-f", multiple=True)
@click.option("--debug", is_flag=True)
@click.option("--crawl", is_flag=True)
@click.option("--output", type=click.Choice(["text", "html"]), default="html")
@click.option("--ignore-type", multiple=True)
@click.option("--setting", "-s", multiple=True)
@click.option("--dir-server-port", type=int, default=None)
@click.option(
    "--config", "-c", type=click.Path(dir_okay=False), default=".sitechecks.yml"
)
def check(
    url,
    debug,
    crawl,
    fingerprint,
    output,
    ignore_type,
    setting,
    dir_server_port,
    config,
):

    if os.path.exists(config):
        click.secho(f"Loading config from {config}", fg="yellow")
        with open(config, "r") as f:
            config_data = yaml.safe_load(f)
            cfg = ConfigSchema().load(config_data)

            # pass the settings on from config
            for k, v in cfg.settings.items():
                Settings[k] = v

    # use command line settings
    for s in setting:
        k, v = s.split("=")
        try:
            value = json.loads(v)
        except json.JSONDecodeError:
            # assume string
            value = v
        key = k.lower()
        Settings[key] = value

    if os.path.exists(url) and os.path.isdir(url):
        click.secho(
            "Looks like you pointed to a directory, starting a server to check it...",
            fg="yellow",
        )
        server = Server(url, port=dir_server_port)
        url = f"http://127.0.0.1:{server.port}"
        click.secho(f"Local site running at {url}", fg="yellow")
        thread = Thread(target=server.serve, daemon=True)
        thread.start()

    if debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    if crawl:
        click.echo(f"Crawling {url}...")
        crawler = Crawler(
            ignore=Settings["crawl-ignore"],
            limit=Settings["crawl-limit"],
            threshold=Settings["crawl-threshold"],
        )
        crawler.crawl(url)
        urls = crawler.urls_found
    else:
        urls = [url]

    runner = Runner(urls, fingerprints=fingerprint, ignore_types=ignore_type)
    runner.run()

    if not runner.urls:
        click.secho("No urls", fg="red")
        exit(1)

    if output == "text":

        failed = False

        for url_runner in runner.url_runners:
            if url_runner.issues:
                click.echo(url_runner.url)
                failed = True
                click.secho(f"{len(url_runner.issues)} issues found", fg="red")
                click.echo()
                for issue_url, url_issues in url_runner.issues.by_url().items():
                    # TODO would be most useful to see these output by line
                    # and if a line contains multiple errors, make that look nice (some may be duplicate depending on check?)
                    click.echo("======================================")
                    click.echo(issue_url)
                    for issue in url_issues:
                        click.secho(issue.type, fg="red")
                        click.echo(issue.fingerprint)
                        click.echo(issue.content)
                        if issue.extra:
                            click.echo(
                                json.dumps(issue.extra, indent=2, ensure_ascii=False)
                            )
                        click.echo()
                    click.echo()
        if failed:
            click.secho("\nⓧ Some checks failed!", fg="red")
            exit(1)
        else:
            click.secho("\n✓ All checks passed!", fg="green")

    elif output == "html":
        report = Report(runner, os.getcwd())
        report_path = report.generate()
        print(f"Report generated and saved at: {report_path}")
        if runner.total_issues:
            click.secho(f"\nⓧ {runner.total_issues} issues found!", fg="red")
            run(["open", report_path])
            exit(1)

        click.secho("\n✓ All checks passed!", fg="green")


if __name__ == "__main__":
    cli()
