import re
import os
from urllib.parse import urlparse, urljoin
from collections import Counter

from bs4 import BeautifulSoup

from . import fetch


ALLOWED_EXTENSIONS = ["", ".html", ".php"]
DEFAULT_CRAWL_LIMIT = 250
DEFAULT_CRAWL_THRESHOLD = 1


class CrawlLimitExceededError(Exception):
    pass


class Crawler:
    def __init__(
        self, ignore=[], limit=DEFAULT_CRAWL_LIMIT, threshold=DEFAULT_CRAWL_THRESHOLD
    ):
        self.ignore = [re.compile(x) for x in ignore]
        self.limit = limit
        self.threshold = threshold
        self._urls_tried = Counter()
        self._urls_found = Counter()

    @property
    def urls_found(self):
        return [
            url for url, count in self._urls_found.items() if count >= self.threshold
        ]

    def should_ignore_url(self, url):
        for pattern in self.ignore:
            if pattern.match(url):
                return True
        return False

    def crawl(self, url):
        if self.should_ignore_url(url):
            return

        response = fetch.get(url)

        # add the original and wherever we ended up
        self._urls_tried.update([url, response.url])

        if len(self._urls_tried) > self.limit:
            raise CrawlLimitExceededError()

        if not response.ok:
            return

        if not url_is_crawlable(response.url):
            return

        if not urls_are_same_site(url, response.url):
            return

        self._urls_found.update([response.url])

        html = response.text
        soup = BeautifulSoup(html, "html.parser")
        links = soup.findAll("a")

        for link in links:
            href = link.get("href")
            if not href:
                continue

            if not href_is_url(href):
                continue

            link_url = urljoin(url, href)

            # https://github.com/seomoz/url-py
            # remove fragment, maybe remove some params too at some point
            link_url = link_url.split("#")[0]

            if urls_are_same_site(url, link_url) and url_is_crawlable(link_url):
                if link_url not in self._urls_tried:
                    self.crawl(link_url)
                elif link_url in self._urls_found:
                    # don't need to crawl it again, but just count it
                    self._urls_found.update([link_url])
                # self._urls_found.update(mo)
                # _urls_found = _urls_found | set(more_urls)
            # elif link_url in self._urls_found:
            #     # if we've already found it, just count it again
            #     self._urls_found.update([link_url])


def urls_are_same_site(a, b):
    return urlparse(a).netloc == urlparse(b).netloc


def url_is_crawlable(u):
    parsed = urlparse(u)

    if ":" in parsed.path:
        # tel:
        return False

    if os.path.splitext(parsed.path)[1] not in ALLOWED_EXTENSIONS:
        return False

    return True


def href_is_url(href):
    if href.startswith("#"):
        return False

    if href.startswith("tel:"):
        return False

    if href.startswith("mailto:"):
        return False

    return True
