import json
import hashlib


ISSUE_TYPES = {
    "a-without-href": {"content": "Anchor tags (almost) always need an href."},
    "a-href-broken": {"content": "Failed to get a valid response for this link."},
    "img-src-broken": {"content": "Failed to load the image source."},
    # "img-alt-empty": {
    #     "content": "Image alt attribute should only be empty if this image has no purpose when it comes to content. You can safely ignore this issue if the image is just a design element.",
    # },
    "img-alt-missing": {
        "content": "Image tag should have an alt attribute for SEO and accessibility purposes. The value of the alt attribute can be empty if the image is purely for visual/design puposes and little or no content value."
    },
    "duplicate-title": {"content": "Multiple pages should not use the same <title>."},
    "short-title": {
        "content": "A one-word page title on pages other than the homepage can sometimes indicate a title that you have forgotten to craft."
    },
    "long-title": {
        "content": "This page title may be too long (more than 60 characters). See https://moz.com/learn/seo/title-tag for more details."
    },
    "title-missing": {"content": "Page title is missing"},
    "duplicate-id": {"content": "Duplicate ID attributes on elements."},
    "http-url-broken": {"content": "The HTTP url failed."},
    "https-redirect-missing": {"content": "The HTTP url should redirect to HTTPS."},
    "http-url": {"content": "You should be using HTTPS."},
    "https-mixed-content": {"content": "Page contains mixed content"},
    "img-filesize-too-large": {
        "content": "Images over 1 MB are likely bigger than they need to be."
    },
    "favicon-missing": {
        "content": "A favicon.ico file should be present at the root of your domain. See https://github.com/audreyr/favicon-cheat-sheet for more details."
    },
    "url-contains-space": {
        "content": 'The url shouldn\'t contain a space or an encoded space ("%20"). Using a hyphen is suggested for separating words.'
    },
    "meta-description-empty": {
        "content": "The meta description tag is present, but the content is empty!"
    },
    "short-meta-description": {
        "content": "A good meta description should be longer than this."
    },
    "long-meta-description": {
        "content": "A description this long will likely get cut off in search results and other places it is used."
    },
    "open-graph-title-missing": {"content": "You really need an open graph title."},
    "open-graph-description-missing": {
        "content": "You need either a meta description tag, or a specific og:description tag."
    },
    "open-graph-type-missing": {"content": "You really need an open graph type."},
    "open-graph-url-missing": {"content": "You really need an open graph url."},
    "open-graph-image-missing": {"content": "You really need an open graph image."},
    "open-graph-site-name-missing": {
        "content": "You really need an open graph site_name."
    },
    "open-graph-url-broken": {"content": "Your og:url does not work."},
    "open-graph-image-broken": {"content": "The url to your og:image does not work."},
    "open-graph-url-not-canonical": {
        "content": "Your og:url needs to be a canonical, absolute http or https url."
    },
    "open-graph-image-not-canonical": {
        "content": "The url to your og:image needs to be a canonical, absolute http or https url."
    },
}


class Issue:
    @classmethod
    def from_type(cls, type, url, **kwargs):
        data = ISSUE_TYPES[type]
        return Issue(url=url, type=type, content=data["content"], **kwargs)

    def __init__(self, url, type, content=None, extra={}):
        self.url = url
        self.type = type

        # Human readable explanation of type?
        self.content = content

        # Instance specific metadata
        self.extra = extra
        assert isinstance(self.extra, dict), "Issue.extra must be a dictionary"
        assert self.extra, "Issue.extra must not be empty"

        self.fingerprint = self.get_fingerprint()

    def get_fingerprint(self):
        fingerprint_data = {"type": self.type, "extra": self.extra}
        dumped = json.dumps(fingerprint_data, sort_keys=True)
        return hashlib.md5(dumped.encode("utf-8")).hexdigest()

    @property
    def extra_for_display(self):
        e = self.extra.copy()
        e.pop("url", None)
        return e

    def as_data(self):
        return {
            "url": self.url,
            "type": self.type,
            "content": self.content,
            "extra": self.extra,
            "fingerprint": self.fingerprint,
        }


class Issues:
    def __init__(self):
        self._issues = []

    # def __init__(self, issues=[]):
    #     self._issues = issues

    def __str__(self):
        return f"{len(self._issues)} issues"

    def __len__(self):
        return len(self._issues)

    def __iter__(self):
        return iter(self._issues)

    def append(self, issue):
        self._issues.append(issue)

    def filter_fingerprints(self, fingerprints):
        self._issues = [x for x in self._issues if x.fingerprint not in fingerprints]

    def filter_types(self, types):
        self._issues = [x for x in self._issues if x.type not in types]

    # def __add__(self, other):
    #     return Issues(self._issues + other._issues)

    def by_url(self):
        out = {}
        for issue in self._issues:
            if issue.url in out:
                out[issue.url].append(issue)
            else:
                out[issue.url] = [issue]
        return out

    def as_data(self):
        return [x.as_data() for x in self._issues]


# class IssueLocation:
# def __init__(self, )
