from urllib.parse import urlparse
from bs4 import BeautifulSoup

from . import fetch
from .issues import Issues, Issue
from .checks import registry


class Runner:
    def __init__(self, urls, fingerprints=[], ignore_types=[]):
        self.urls = sorted(urls)
        self.fingerprints = fingerprints
        self.ignore_types = ignore_types

        self.url_runners = []

    def run(self):
        self.url_runners = []

        for url in self.urls:
            runner = URLRunner(url)
            runner.run()
            if self.fingerprints:
                runner.issues.filter_fingerprints(self.fingerprints)
            if self.ignore_types:
                runner.issues.filter_types(self.ignore_types)
            self.url_runners.append(runner)

    @property
    def total_issues(self):
        return sum([len(x.issues) for x in self.url_runners])


class URLRunner:
    """Runner processes a single URL"""

    def __init__(self, url):
        self.url = url
        self.properties = {}
        self.content = ""
        self.status_code = None
        self.issues = Issues()

    def get_url_content(self, url):
        response = fetch.get(url)

        self.status_code = response.status_code
        self.url = response.url
        self.path = urlparse(url).path
        self.content = response.text

        soup = BeautifulSoup(self.content, "html.parser")

        def get_meta(props):
            found = soup.find("meta", props)
            if found:
                return found.get("content", "")
            return ""

        self.properties = {
            "title": soup.title.text if soup.title else "",
            "description": get_meta({"name": "description"}),
            "og:title": get_meta({"property": "og:title"}),
            "og:description": get_meta({"property": "og:description"}),
            # "og:type": get_meta({"property": "og:type"}),
            # "og:url": get_meta({"property": "og:url"}),
            "og:image": get_meta({"property": "og:image"}),
            "og:site_name": get_meta({"property": "og:site_name"}),
        }

        response.raise_for_status()

    def run(self):
        print(f"Checking {self.url}")
        try:
            self.get_url_content(self.url)
        except Exception as e:
            self.issues.append(
                Issue(
                    url=self.url,
                    type="request-failed",
                    content="Failed to load the url.",
                    extra={"exception": str(e)},
                )
            )
        else:
            for cls in registry.registered_checks:
                check = cls(self.url, self.content)
                check_issues = check.run()
                # all_issues = all_issues + check_issues
                self.issues._issues += check_issues._issues

        # TODO print issues as they are found... want to see that in output
